==> libinklevel-0.6.3.tar.gz: libinklevel/epson.c <==
/* epson.c
 *
 * (c) 2003, 2004 Markus Heinz
 *
 * Code taken from escputil (gimp-print).
 *
 * This software is licensed under the terms of the GPL.
 * For details see file COPYING.
 */

#include "inklevel.h"

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>

extern int read_from_printer(int, void *, size_t);
extern int my_axtoi(unsigned char *);

int get_ink_level_epson(int, int, int, struct ink_level *);

/* This funtion retrieves the ink level of an Epson printer conncected to
 * the specifies port and portnumber
 */

int get_ink_level_epson(int port, int portnumber, int type,
			       struct ink_level *level) {
  unsigned char device_file[256];
  int fd;
  unsigned char hdr1[] = "\000\000\000\033\001@EJL 1284.4\n@EJL     \n\033@";
  unsigned char hdr2[] = "\033@\033(R\010\000\000REMOTE1";
  unsigned char cmd[] = "ST\002\000\000\001";
  unsigned char trl[] = "\033\000\000\000\033\000";
  unsigned char rst[] = "\033\000\033\000";
  unsigned char command[256];
  int length;
  int i = 0;
  unsigned char buffer[BUFLEN];
  unsigned char *index = NULL;
  unsigned char number[3];
  int current_pos = 0;

  if (port == USB) {
    sprintf(device_file, "/dev/usb/lp%d", portnumber);
  } else if (port == PARPORT) {
    sprintf(device_file, "/dev/lp%d", portnumber);
  } else {
    return UNKNOWN_PORT_SPECIFIED;
  }

#ifdef DEBUG
  printf("Device file: %s\n", device_file);
#endif

  fd = open(device_file, O_RDWR, 0666);
  if (fd == -1) {

#ifdef DEBUG
    printf("Could not open %s\n", device_file);
#endif

    if (port == USB) {
      return DEV_USB_LP_INACCESSIBLE;
    } else { 
      return DEV_LP_INACCESSIBLE;
    }
  }

  if (type == NEW) { /* we have to exit packet mode first */
    memcpy(command, hdr1, sizeof(hdr1) - 1);
    current_pos += (sizeof(hdr1) - 1);
  }

  memcpy(command + current_pos, hdr2, sizeof(hdr2) - 1);
  current_pos += (sizeof(hdr2) - 1);

  memcpy(command + current_pos, cmd, sizeof(cmd) - 1);
  current_pos += (sizeof(cmd) - 1);

  memcpy(command + current_pos, trl, sizeof(trl) - 1);
  current_pos += (sizeof(trl) - 1);

  memcpy(command + current_pos, rst, sizeof(rst) - 1);
  current_pos += (sizeof(rst) - 1);

  length = current_pos;

#ifdef DEBUG
  printf("Command: \n");
  for(i = 0; i < length; i++) {
    printf("%#4o\n", command[i]);
  }
#endif

  i = write(fd, command, length);
  if (i < length) {

#ifdef DEBUG
    printf("Could not send command to printer\n");
#endif

    close(fd);
    return COULD_NOT_WRITE_TO_PRINTER;
  }

  length = read_from_printer(fd, buffer, BUFLEN);
  if (length <= 0) {

#ifdef DEBUG    
    printf("Could not read from printer\n");
#endif

    close(fd);
    return COULD_NOT_READ_FROM_PRINTER;
  }

#ifdef DEBUG
  printf("Printer returned %s\n", buffer);
#endif

  close(fd);

  index = buffer;

  do {
    index = strchr(index, 'I');
  } while ((index) && (index[1] != 'Q') && (index[1] != '\0') && 
	   (index[2] != ':'));

  if ((!index) || (index[1] != 'Q') || (index[2] != ':')) {

#ifdef DEBUG
    printf("Could not parse output from printer\n");
#endif

    return COULD_NOT_PARSE_RESPONSE_FROM_PRINTER;
  }

  index += 3;

  if ((index[0]) && (index[0] != ';')) {
    level->type = ONE_BLACK_COLOR_FOUND;
    number[0] = index[0];
    number[1] = index[1];
    number[2] = '\0';
    level->black = my_axtoi(number);

#ifdef DEBUG
    printf("Black: %d%%\n", level->black);
#endif

    index += 2;
  } else {

#ifdef DEBUG
    printf("No ink level found\n");
#endif

    return NO_INK_LEVEL_FOUND;
  }

  if ((index[0]) && (index[0] != ';')) {
    level->type = FOUR_COLORS_FOUND;
    number[0] = index[0];
    number[1] = index[1];
    number[2] = '\0';
    level->cyan = my_axtoi(number);

#ifdef DEBUG
    printf("Cyan: %d%%\n", level->cyan);
#endif

    index += 2;
  }

  if ((index[0]) && (index[0] != ';')) {
    number[0] = index[0];
    number[1] = index[1];
    number[2] = '\0';
    level->magenta = my_axtoi(number);

#ifdef DEBUG
    printf("Magenta: %d%%\n", level->magenta);
#endif

    index += 2;
  }

  if ((index[0]) && (index[0] != ';')) {
    number[0] = index[0];
    number[1] = index[1];
    number[2] = '\0';
    level->yellow = my_axtoi(number);

#ifdef DEBUG
    printf("Yellow: %d%%\n", level->yellow);
#endif

    index += 2;
  }

  if ((index[0]) && (index[0] != ';')) {
    level->type = SIX_COLORS_FOUND;
    number[0] = index[0];
    number[1] = index[1];
    number[2] = '\0';
    level->photocyan = my_axtoi(number);

#ifdef DEBUG
    printf("Photocyan: %d%%\n", level->photocyan);
#endif

    index += 2;
  }
  if ((index[0]) && (index[0] != ';')) {
    number[0] = index[0];
    number[1] = index[1];
    number[2] = '\0';
    level->photomagenta = my_axtoi(number);

#ifdef DEBUG
    printf("Photomagenta: %d%%\n", level->photomagenta);
#endif

    index += 2;
  }
  if ((index[0]) && (index[0] != ';')) {
    level->type = SEVEN_COLORS_FOUND;
    number[0] = index[0];
    number[1] = index[1];
    number[2] = '\0';
    level->photoyellow = my_axtoi(number);

#ifdef DEBUG
    printf("Photoyellow: %d%%\n", level->photoyellow);
#endif

  }

  return OK;
}

==> libinklevel-0.6.5rc1.tar.gz: libinklevel/epson.c <==
/* epson.c
 *
 * (c) 2003, 2004, 2005 Markus Heinz
 *
 * Code taken from escputil (gimp-print).
 *
 * This software is licensed under the terms of the GPL.
 * For details see file COPYING.
 */

#include "inklevel.h"

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>

extern int read_from_printer(int fd, void *buf, size_t bufsize);
extern int my_axtoi(unsigned char *t);

int get_ink_level_epson(int port, int portnumber, int type, 
                        struct ink_level *level);

/* This funtion retrieves the ink level of an Epson printer conncected to
 * the specifies port and portnumber
 */

int get_ink_level_epson(int port, int portnumber, int type,
                        struct ink_level *level) {
  unsigned char device_file[256];
  int fd;
  unsigned char hdr1[] = "\000\000\000\033\001@EJL 1284.4\n@EJL     \n\033@";
  unsigned char hdr2[] = "\033@\033(R\010\000\000REMOTE1";
  unsigned char cmdST[] = "ST\002\000\000\001";
  unsigned char cmdIQ[] = "IQ\001\000\001";
  unsigned char trl[] = "\033\000\000\000\033\000";
  unsigned char rst[] = "\033\000\033\000";
  unsigned char command[256];
  int length;
  int i = 0;
  unsigned char buffer[BUFLEN];
  unsigned char *index = NULL;
  unsigned char number[3];
  int current_pos = 0;
  int retry = 6; /* You can change this, but keep same parity */

  if (port == USB) {
    sprintf(device_file, "/dev/usb/lp%d", portnumber);
  } else if (port == PARPORT) {
    sprintf(device_file, "/dev/lp%d", portnumber);
  } else {
    return UNKNOWN_PORT_SPECIFIED;
  }

#ifdef DEBUG
  printf("Device file: %s\n", device_file);
#endif

  do {
    fd = open(device_file, O_RDWR, 0666);
    if (fd == -1) {
	  
#ifdef DEBUG
      printf("Could not open %s\n", device_file);
#endif

      if (port == USB) {
        return DEV_USB_LP_INACCESSIBLE;
      } else { 
        return DEV_LP_INACCESSIBLE;
      }
    }

    memcpy(command, rst, sizeof(rst) - 1);
    current_pos = (sizeof(rst) - 1);

    if (type == NEW) { /* we have to exit packet mode first */
      memcpy(command + current_pos, hdr1, sizeof(hdr1) - 1);
      current_pos += (sizeof(hdr1) - 1);
    }

    memcpy(command + current_pos, hdr2, sizeof(hdr2) - 1);
    current_pos += (sizeof(hdr2) - 1);
      
    /*
     * Some new printers like IQ and others like ST for retrieving status.
     * Some printers will take either, but not reliably in either
     * case. In some cases, alternating between the two works best.
     */

    if (type == NEW && !(retry & 1)) {
      memcpy(command + current_pos, cmdIQ, sizeof(cmdIQ) - 1);
      current_pos += (sizeof(cmdIQ) - 1);
    } else {
      memcpy(command + current_pos, cmdST, sizeof(cmdST) - 1);
      current_pos += (sizeof(cmdST) - 1);
    }

    memcpy(command + current_pos, trl, sizeof(trl) - 1);
    current_pos += (sizeof(trl) - 1);

    memcpy(command + current_pos, rst, sizeof(rst) - 1);
    current_pos += (sizeof(rst) - 1);

    length = current_pos;

#ifdef DEBUG
    printf("Command: \n");
    for(i = 0; i < length; i++) {
      printf("%#4o\n", command[i]);
    }
#endif

    i = write(fd, command, length);
    if (i < length) {

#ifdef DEBUG
      printf("Could not send command to printer\n");
#endif

      close(fd);
      return COULD_NOT_WRITE_TO_PRINTER;
    }

    length = read_from_printer(fd, buffer, BUFLEN);
    close(fd);
    if (length <= 0) {

#ifdef DEBUG    
      printf("Could not read from printer\n");
#endif
      return COULD_NOT_READ_FROM_PRINTER;
    }

#ifdef DEBUG
    printf("Printer returned <<%s>>\n", buffer);
#endif
      
    index = strstr(buffer, "IQ:");
  } while (!index && --retry);
  if (!index) {
	  
#ifdef DEBUG
    printf("Could not parse output from printer\n");
#endif

    return COULD_NOT_PARSE_RESPONSE_FROM_PRINTER;
  }

  index += 3;

  if ((index[0]) && (index[0] != ';')) {
    level->type = ONE_BLACK_COLOR_FOUND;
    number[0] = index[0];
    number[1] = index[1];
    number[2] = '\0';
    level->black = my_axtoi(number);

#ifdef DEBUG
    printf("Black: %d%%\n", level->black);
#endif

    index += 2;
  } else {
      
#ifdef DEBUG
    printf("No ink level found\n");
#endif

    return NO_INK_LEVEL_FOUND;
  }
  
  if ((index[0]) && (index[0] != ';')) {
    level->type = FOUR_COLORS_FOUND;
    number[0] = index[0];
    number[1] = index[1];
    number[2] = '\0';
    level->cyan = my_axtoi(number);
      
#ifdef DEBUG
    printf("Cyan: %d%%\n", level->cyan);
#endif
      
    index += 2;
  }
  
  if ((index[0]) && (index[0] != ';')) {
    number[0] = index[0];
    number[1] = index[1];
    number[2] = '\0';
    level->magenta = my_axtoi(number);
      
#ifdef DEBUG
    printf("Magenta: %d%%\n", level->magenta);
#endif
      
    index += 2;
  }

  if ((index[0]) && (index[0] != ';')) {
    number[0] = index[0];
    number[1] = index[1];
    number[2] = '\0';
    level->yellow = my_axtoi(number);
      
#ifdef DEBUG
    printf("Yellow: %d%%\n", level->yellow);
#endif

    index += 2;
  }
  
  if ((index[0]) && (index[0] != ';')) {
    level->type = SIX_COLORS_FOUND;
    number[0] = index[0];
    number[1] = index[1];
    number[2] = '\0';
    level->photocyan = my_axtoi(number);
      
#ifdef DEBUG
    printf("Photocyan: %d%%\n", level->photocyan);
#endif
      
    index += 2;
  }

  if ((index[0]) && (index[0] != ';')) {
    number[0] = index[0];
    number[1] = index[1];
    number[2] = '\0';
    level->photomagenta = my_axtoi(number);
      
#ifdef DEBUG
    printf("Photomagenta: %d%%\n", level->photomagenta);
#endif

    index += 2;
  }

  if ((index[0]) && (index[0] != ';')) {
    level->type = SEVEN_COLORS_FOUND;
    number[0] = index[0];
    number[1] = index[1];
    number[2] = '\0';
    level->photoyellow = my_axtoi(number);
      
#ifdef DEBUG
    printf("Photoyellow: %d%%\n", level->photoyellow);
#endif
      
  }
  
  return OK;
}
